package com.sos.example;

import javax.jms.Connection;
import javax.jms.ConnectionFactory;
import javax.jms.Destination;
import javax.jms.JMSException;
import javax.jms.Message;
import javax.jms.MessageConsumer;
import javax.jms.MessageProducer;
import javax.jms.Session;
import javax.jms.TextMessage;

import org.apache.activemq.ActiveMQConnectionFactory;
import org.apache.log4j.Logger;
import org.junit.Test;

import static org.junit.Assert.*;

public class TestActiveMq {

    private static final String URI = "tcp://galadriel.sos:61616";
    private static final String QUEUE_NAME = "test_queue";
    private static final String JUNIT_QUEUE_NAME = "junit_test_queue";
    private static final String TEXT = "Hello JUnit World!";
    private static final Logger LOGGER = Logger.getLogger(TestActiveMq.class);
    private ConnectionFactory factory = null;
    private Connection connection = null;

    private void setUp(){
        factory = new ActiveMQConnectionFactory(URI);
        try {
            connection = factory.createConnection();
        } catch (JMSException e) {
            LOGGER.error("JMSException occurred: " , e);
        }
        
    }
    
    @Test
    public void testWriteToBroker(){
        setUp();
        Session session = null;
        Destination destination = null;
        MessageProducer producer = null;
        try {
            session = connection.createSession(false, Session.AUTO_ACKNOWLEDGE);
            destination = session.createQueue(JUNIT_QUEUE_NAME);
            LOGGER.info("destination " + destination.toString() + " created!");
            producer = session.createProducer(destination);
            Message message = session.createTextMessage(TEXT);
            producer.send(message);
        } catch (JMSException e) {
            LOGGER.error("JMSException occurred: " , e);
        }
        assertNotNull("Session created sucessfully!", session);
        assertNotNull("Destination created sucessfully!", session);
        assertNotNull("Producer created sucessfully!", session);
        
    }
     
    @Test
    public void testReadFromBroker(){
        setUp();
        Session session = null;
        Destination destination = null;
        MessageConsumer consumer = null;
        TextMessage message = null;
        String textMessage = null;
        
        try {
            session = connection.createSession(false, Session.AUTO_ACKNOWLEDGE);
            destination = session.createQueue(JUNIT_QUEUE_NAME);
            consumer = session.createConsumer(destination);
            connection.start();
            while (true) {
                Message m = consumer.receive(1);
                if (m != null) {
                    if (m instanceof TextMessage) {
                        message = (TextMessage)m;
                        textMessage = message.getText();
                        LOGGER.info("Reading message: " + textMessage);
                        break;
                    } else {
                        break;
                    }
                }
            }
        } catch (JMSException e) {
            LOGGER.error("JMSException occurred: " , e);
        } finally {
            assertEquals(TEXT, textMessage);
            if (connection != null) {
                try {
                    connection.close();
                } catch (JMSException e) {
                    LOGGER.error("JMSException occurred while trying to close the connection: " , e);
                }
            }
        }
       
    }
}
